
package org.mule.module.google.calendar.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.google.calendar.processors.BatchUpdateCalendarMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class BatchUpdateCalendarDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public BatchUpdateCalendarDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(BatchUpdateCalendarMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accessToken")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessToken")))) {
            builder.addPropertyValue("accessToken", element.getAttribute("accessToken"));
        }
        Element calendarsListElement = null;
        calendarsListElement = DomUtils.getChildElementByTagName(element, "calendars");
        List<Element> calendarsMapChilds = null;
        if (calendarsListElement!= null) {
            String calendarsRef = calendarsListElement.getAttribute("ref");
            if ((calendarsRef!= null)&&(!StringUtils.isBlank(calendarsRef))) {
                if ((!calendarsRef.startsWith(patternInfo.getPrefix()))&&(!calendarsRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("calendars", new RuntimeBeanReference(calendarsRef));
                } else {
                    builder.addPropertyValue("calendars", calendarsRef);
                }
            } else {
                ManagedMap calendars = new ManagedMap();
                calendarsMapChilds = DomUtils.getChildElementsByTagName(calendarsListElement, "calendar");
                if (calendarsMapChilds!= null) {
                    if (calendarsMapChilds.size() == 0) {
                        calendarsMapChilds = DomUtils.getChildElements(calendarsListElement);
                    }
                    for (Element calendarsChild: calendarsMapChilds) {
                        String calendarsValueRef = calendarsChild.getAttribute("value-ref");
                        String calendarsKeyRef = calendarsChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((calendarsValueRef!= null)&&(!StringUtils.isBlank(calendarsValueRef))) {
                            if ((!calendarsValueRef.startsWith(patternInfo.getPrefix()))&&(!calendarsValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(calendarsValueRef);
                            } else {
                                valueObject = calendarsValueRef;
                            }
                        } else {
                            valueObject = calendarsChild.getTextContent();
                        }
                        if ((calendarsKeyRef!= null)&&(!StringUtils.isBlank(calendarsKeyRef))) {
                            keyObject = new RuntimeBeanReference(calendarsKeyRef);
                        } else {
                            keyObject = calendarsChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = calendarsChild.getTagName();
                        }
                        calendars.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("calendars", calendars);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
