
package org.mule.module.google.calendar.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.google.calendar.processors.BatchUpdateEventMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class BatchUpdateEventDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public BatchUpdateEventDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(BatchUpdateEventMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accessToken")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessToken")))) {
            builder.addPropertyValue("accessToken", element.getAttribute("accessToken"));
        }
        if ((element.getAttribute("calendarId")!= null)&&(!StringUtils.isBlank(element.getAttribute("calendarId")))) {
            builder.addPropertyValue("calendarId", element.getAttribute("calendarId"));
        }
        Element eventsListElement = null;
        eventsListElement = DomUtils.getChildElementByTagName(element, "events");
        List<Element> eventsMapChilds = null;
        if (eventsListElement!= null) {
            String eventsRef = eventsListElement.getAttribute("ref");
            if ((eventsRef!= null)&&(!StringUtils.isBlank(eventsRef))) {
                if ((!eventsRef.startsWith(patternInfo.getPrefix()))&&(!eventsRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("events", new RuntimeBeanReference(eventsRef));
                } else {
                    builder.addPropertyValue("events", eventsRef);
                }
            } else {
                ManagedMap events = new ManagedMap();
                eventsMapChilds = DomUtils.getChildElementsByTagName(eventsListElement, "event");
                if (eventsMapChilds!= null) {
                    if (eventsMapChilds.size() == 0) {
                        eventsMapChilds = DomUtils.getChildElements(eventsListElement);
                    }
                    for (Element eventsChild: eventsMapChilds) {
                        String eventsValueRef = eventsChild.getAttribute("value-ref");
                        String eventsKeyRef = eventsChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((eventsValueRef!= null)&&(!StringUtils.isBlank(eventsValueRef))) {
                            if ((!eventsValueRef.startsWith(patternInfo.getPrefix()))&&(!eventsValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(eventsValueRef);
                            } else {
                                valueObject = eventsValueRef;
                            }
                        } else {
                            valueObject = eventsChild.getTextContent();
                        }
                        if ((eventsKeyRef!= null)&&(!StringUtils.isBlank(eventsKeyRef))) {
                            keyObject = new RuntimeBeanReference(eventsKeyRef);
                        } else {
                            keyObject = eventsChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = eventsChild.getTagName();
                        }
                        events.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("events", events);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
